==> series-1.0.1.tar.gz: series-1.0.1/series/daemon/feed_poller.py <==
__copyright__ = """ Copyright (c) 2013 Torsten Schmits

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see <http://www.gnu.org/licenses/>.
"""

import urllib2
import pickle

import feedparser

from xml.sax import SAXException

from tek import logger
from tek.config import lazy_configurable
from tek.tools import find

from series.daemon.handler import Handler
from series.daemon.model.feed_entry import FeedEntryFactory
from series.daemon.model.release import ReleaseFactory

@lazy_configurable(series=['monitor'],
                   daemon=['rss_urls', 'rss_interval'])
class FeedPoller(Handler):

    def __init__(self, session, queue, commit):
        self._session = session
        super(FeedPoller, self).__init__(queue, commit, self._rss_interval,
                                         'feed poller')
        self._rss = None
        self._etags = [None] * len(self._rss_urls)
        self._modified = [None] * len(self._rss_urls)
        self._release_fact = ReleaseFactory()

    def reconstruct_entries(func):
        def wrapper(dl, *a, **kw):
            ret = func(dl, *a, **kw)
            dl._reconstruct_entries()
            return ret
        return wrapper

    @reconstruct_entries
    def load_from_pickle_file(self, fname):
        with open(fname) as pfile:
            self._rss = pickle.load(pfile)
        text = 'Unpickled {} rss entries from "{}".'
        logger.info(text.format(len(self._rss), fname))

    @reconstruct_entries
    def load_from_xml_file(self, fname):
        try:
            self._rss = feedparser.parse(fname)
        except OSError as e:
            logger.error(e)

    @reconstruct_entries
    def load_from_url(self, url, index):
        try:
            self._rss = feedparser.parse(url, etag=self._etags[index],
                                         modified=self._modified[index])
            self._etags[index] = self._rss.get('etag', None)
            self._modified[index] = self._rss.get('modified_parsed', None)
        except (urllib2.URLError, urllib2.HTTPError, SAXException) as e:
            logger.debug(e)

    def _handle(self, ignore):
        for index, url in enumerate(self._rss_urls):
            self.load_from_url(url, index)

    @property
    def _current(self):
        return True

    def save_to_pickle_file(self, fname):
        with open(fname, 'w') as pfile:
            pickle.dump(self._rss, pfile)

    def _reconstruct_entries(self):
        if self._rss and self._rss.entries:
            fact = FeedEntryFactory(self._rss.feed.link)
            self._feed_entries = list(fact.series(reversed(self._rss.entries)))
            text = u'Found {} series entries in the feed.'
            logger.info(text.format(len(self._feed_entries)))
            self._update_monitors()

    def _update_monitors(self):
        is_monitored = lambda e: e.release.name in self._monitor
        for entry in filter(is_monitored, self._feed_entries):
            existing = self._find_release(entry)
            if existing is None:
                self._add_monitor(entry)
            else:
                existing.add_links_from_feed_entry(entry)

    def _add_monitor(self, entry):
        monitor = self._release_fact.monitor(entry, len(self._queue))
        if entry.is_fix:
            logger.debug(u'fix: {!r}'.format(entry.release))
            old_entry = self._find_nuke(entry)
            if old_entry is not None:
                old_entry.nuke(monitor)
        self._session.add_then_commit(monitor)
        self._queue.append(monitor)
        logger.info(u'Added monitor for {!s}'.format(monitor.release))

    def _find_release(self, entry):
        return find(lambda m: m.release == entry.release, self._queue)

    def _find_nuke(self, entry):
        return find(lambda m: m.release.is_same_episode(entry.release),
                    self._queue)

__all__ = ['FeedPoller']

==> series-1.1.1.tar.gz: series-1.1.1/series/daemon/feed_poller.py <==
import urllib.error
import pickle

import feedparser

from xml.sax import SAXException

from tek import logger
from tek.config import lazy_configurable
from tek.tools import find

from series.daemon.handler import Handler
from series.daemon.model.feed_entry import FeedEntryFactory
from series.daemon.model.feed_entry import FeedEntryHasher
from series.daemon.model.release import ReleaseFactory

feedparser.PREFERRED_XML_PARSERS = []


@lazy_configurable(series=['monitor'],
                   daemon=['rss_urls', 'rss_interval'])
class FeedPoller(Handler):

    def __init__(self, session, queue, commit):
        self._session = session
        super(FeedPoller, self).__init__(queue, commit, self._rss_interval,
                                         'feed poller')
        self._rss = None
        self._etags = [None] * len(self._rss_urls)
        self._modified = [None] * len(self._rss_urls)
        self._release_fact = ReleaseFactory()
        self._hashes = set()

    def reconstruct_entries(func):
        def wrapper(dl, *a, **kw):
            ret = func(dl, *a, **kw)
            dl._reconstruct_entries()
            return ret
        return wrapper

    @reconstruct_entries
    def load_from_pickle_file(self, fname):
        with open(fname) as pfile:
            self._rss = pickle.load(pfile)
        text = 'Unpickled {} rss entries from "{}".'
        logger.info(text.format(len(self._rss), fname))

    @reconstruct_entries
    def load_from_xml_file(self, fname):
        try:
            self._rss = feedparser.parse(fname)
        except OSError as e:
            logger.error(e)

    @reconstruct_entries
    def load_from_url(self, url, index):
        try:
            self._rss = feedparser.parse(url, etag=self._etags[index],
                                         modified=self._modified[index])
            self._etags[index] = self._rss.get('etag', None)
            self._modified[index] = self._rss.get('modified_parsed', None)
        except (urllib.error.URLError, urllib.error.HTTPError,
                SAXException) as e:
            logger.debug(e)

    def _handle(self, ignore):
        for index, url in enumerate(self._rss_urls):
            self.load_from_url(url, index)

    @property
    def _current(self):
        return True

    def save_to_pickle_file(self, fname):
        with open(fname, 'w') as pfile:
            pickle.dump(self._rss, pfile)

    def _reconstruct_entries(self):
        if self._rss and self._rss.entries:
            hasher = FeedEntryHasher(self._rss.feed.link)
            valid = lambda e: (bool(e) and not hasher(e) in self._hashes)
            valid_entries = list(filter(valid, self._rss.entries))
            self._hashes |= set([hasher(e) for e in valid_entries])
            fact = FeedEntryFactory(self._rss.feed.link)
            self._feed_entries = list(fact.series(reversed(valid_entries)))
            text = 'Found {} series entries in the feed.'
            logger.info(text.format(len(valid_entries)))
            self._update_monitors()

    def _update_monitors(self):
        is_monitored = lambda e: e.release.name in self._monitor
        for entry in filter(is_monitored, self._feed_entries):
            existing = self._find_release(entry)
            if existing is None:
                self._create_monitor(entry)
            else:
                existing.add_links_from_feed_entry(entry)

    def _create_monitor(self, entry):
        monitor = self._release_fact.monitor(entry, len(self._queue))
        old_entry = self._find_nuke(entry)
        if old_entry is not None:
            if not entry.is_fix:
                monitor.downloaded = old_entry.downloaded
                monitor.archived = old_entry.archived
            old_entry.nuke(monitor)
        self._session.add_then_commit(monitor)
        self._queue.append(monitor)
        logger.info('Added monitor for {!s}'.format(monitor.release))

    def _find_release(self, entry):
        return find(lambda m: m.release == entry.release, self._queue)

    def _find_nuke(self, entry):
        return find(lambda m: m.release.is_same_episode(entry.release),
                    self._queue)

__all__ = ['FeedPoller']
